/**
 * Integrate CSScomb into Webuilder
 *
 * CSScomb is a coding style formatter for CSS. <http://www.csscomb.com/>
 * You can easily write your own configuration to make your style sheets beautiful and consistent.
 * Requires nodejs. <https://nodejs.org/>
 *
 * @category  WeBuilder Plugin
 * @package   CSScomb Integration
 * @author    Peter Klein <pmk@io.dk>
 * @copyright 2017
 * @license   http://www.freebsd.org/copyright/license.html  BSD License
 * @version   2.02
 */

/**
 * [CLASS/FUNCTION INDEX of SCRIPT]
 *
 *     51   function InitCSSComb()
 *     65   function ValidateDocCodeTypes()
 *    128   function ShowBusyPanel(type)
 *    171   function BusyPanelOnShow()
 *    182   function StartCombing()
 *    236   function FileGetContents(filename)
 *    256   function FilePutContents(filename, contents)
 *    272   function OnInstalled()
 *
 * TOTAL FUNCTIONS: 8
 * (This index is automatically created/updated by the WeBuilder plugin "DocBlock Comments")
 *
 */

/**
 * Global variables
 *
 */
var combedCssText,
	textToComb,
	outputText,
	PSForm,
	curText,
	curCmd,
	curDocType,
	curCodeType;

/**
 * Setup variables for CSS Beautifying
 *
 * @return void
 */
function InitCSSComb() {
	curText = "CSS";
	curCmd = "csscomb.cmd";
	curCfg = "csscomb.json";
	curDocType = [dtCSS, dtSass, dtLess];
	curCodeType = [ltCSS];
	ValidateDocCodeTypes();
}

/**
 * Check if document or selection codetype matches the beautyfier selected
 *
 * @return void
 */
function ValidateDocCodeTypes() {

	var enc = Document.Encoding;

	// Is something selected?
	if (Editor.SelText !="") {
		if ( Document.CurrentCodeType in curCodeType ) {

			// Workaround for encoding problem
			Editor.BeginEditing;
			if ((Script.ReadSetting("Convert Unicode to Entities", "0") == "1") && (enc == 0)) {
				Actions.Execute("ActCharsToEntities");
			}

			textToComb = Editor.SelText;
			type = "Selection";
		}
		else {
			Alert("Error: Selection is not " + curText + "!");
			return;
		}
	}
	// If not we grab the entire document
	else {
		if (Document.DocType in curDocType) {

			// Workaround for encoding problem
			Editor.BeginEditing;
			if ((Script.ReadSetting("Convert Unicode to Entities", "0") == "1") && (enc == 0)) {

				// Save selection/scroll position
				var sel = Editor.Selection,
					topLine = Editor.TopLine;

				Actions.Execute("ActSelectAll");
				Actions.Execute("ActCharsToEntities");

				// Restore selection/scroll position
				Editor.Selection = sel;
				Editor.TopLine = topLine;
			}

			textToComb = Editor.Text;
			type = "Document";
		}
		else {
			Alert("Error: File is not " + curText + "!");
			return;
		}
	}
	// Show busy panel and run csscomb
	ShowBusyPanel(type);
}

/**
 * Display Busy panel.
 * Prevents user from doing other actions while content is being beautified
 *
 * @param  string   type "Document" or "Selection"
 *
 * @return void
 */

function ShowBusyPanel(type) {

	PSForm = new TForm(WeBuilder);
	PSForm.Height = 70;
	PSForm.Width = 320;
	PSForm.Position = poScreenCenter;
	PSForm.BorderStyle = bsNone;  // disable dialog resizing
	PSForm.BorderIcons = 0; // remove close, maximize & minimize buttons
	PSForm.FormStyle = fsStayOnTop;
	PSForm.Color = clBtnFace;
	PSForm.Font.Color = clWindowText;
	PSForm.Font.Name = "Tahoma";
	PSForm.Font.Height = -21;
	PSForm.PixelsPerInch = 120;
	PSForm.OnShow = "BusyPanelOnShow";

	Panel1 = new TPanel(WeBuilder);
	Panel1.Parent = PSForm;
	Panel1.Align = alClient;

	Image1 = new TImage(WeBuilder);
	Image1.Parent = Panel1;
	Image1.SetBounds(10, 12, 45, 45);
    Image1.Picture.LoadFromFile(Script.Path + "/comb.png");

	Label1 = new TLabel(WeBuilder);
	Label1.Parent = Panel1;
	Label1.Layout = tlCenter;
	Label1.Alignment = taCenter;
	Label1.Caption = "Combing " + type + chr(13) + "Please Wait...";
	Label1.SetBounds(65, 0, 255, 70);

	PSForm.ShowModal;

	delete PSForm;

}

/**
 * Callback function for when the Busy Panel is displayed
 *
 * @return void
 */
function BusyPanelOnShow() {
	// We need a time delay, otherwise the closing of the modal window will be ignored.
	Script.TimeOut(1, "StartCombing");
}


/**
 * Integrate CSScomb into Webuilder
 *
 * @return void
 */
function StartCombing() {
	var fileName = Document.FileName,	// Filename of current document
		ext = ExtractFileExt(fileName);	// File extension of current document

	// if name of file in Editor is "styles.css", work filename will be "styles-wbcwork.css"
	var workFileName = RegexReplace(fileName, Replace(ext,".","\\.") + "$", "", false) + "-wbcwork" + ext;

	// Remove existing *-wbcwork file to avoid double requesters popping up
	if (FileExists(workFileName)) {
	 	ExecuteCommand(Script.Path + "deletefile.cmd \"" + workFileName + "\"", null);
	}

	// Save a copy of original document
	FilePutContents(workFileName, textToComb);

	if (FileExists(workFileName)) {
		// If file exists (saved correctly), then we run CSScomb
		var cmd = Script.Path + "csscomb.cmd -v -c \"" + Script.Path + "csscomb.json\" \"" + workFileName + "\"";
		if (ExecuteCommand(cmd, outputText)) {
			// Grab the content of the processed workfile
			combedCssText = FileGetContents(workFileName);

			// Remove the work file as we no longer need it
		 	ExecuteCommand(Script.Path + "deletefile.cmd \"" + workFileName + "\"", null);

			// Replace editor content with combed content
			//Editor.BeginEditing;	// Moved to ValidateDocCodeTypes, in order to capture ActCharsToEntities
			if ((Editor.SelText !="")) Editor.SelText = combedCssText;
			else Editor.Text = combedCssText;
			Editor.EndEditing;

			PSForm.ModalResult = mrCancel;	// Close modal window
		}
		else {
    		PSForm.ModalResult = mrCancel;	// Close modal window
			Alert("An error has occured. Unable to run CSScomb.");
		}
	}
	else {
			PSForm.ModalResult = mrCancel;	// Close modal window
			Alert("An error has occured. Was not able to save workfile.");
	}

	// Just for safety
    PSForm.ModalResult = mrCancel;	// Close modal window
}

/**
 * Load external file and return the contents
 *
 * @param  string   filename the name of the file to load
 *
 * @return string  the contents of the file
 */
function FileGetContents(filename) {
    var stream = CreateOleObject("ADODB.Stream");
    stream.Open;
    stream.Position = 0;
    stream.CharSet = "utf-8";
    stream.Type = 2;
    stream.LoadFromFile(filename);
    var contents = stream.ReadText();
    stream.Close;
	return contents;
}

/**
 * Save contents to external file
 *
 * @param  string   filename the name of the file to save
 * @param  string   contents the contents of the file
 *
 * @return void
 */
function FilePutContents(filename, contents) {
    var stream = CreateOleObject("ADODB.Stream");
    stream.Open;
    stream.Position = 0;
    stream.CharSet = "utf-8";
    stream.Type = 2;
    stream.WriteText(contents);
    stream.SaveToFile(filename, 2);
    stream.Close;
}

/**
 * Show info when plugin is installed
 *
 * @return void
 */
function OnInstalled() {
  alert("CSScombWB 2.02 by Peter Klein installed sucessfully!");
}

Script.ConnectSignal("installed", "OnInstalled");

var bmp = new TBitmap, act;
LoadFileToBitmap(Script.Path + "comb_icon.png", bmp);
act = Script.RegisterDocumentAction("", "CSScomb", "", "InitCssComb");
Actions.SetIcon(act, bmp);
delete bmp;